@@ -49,7 +49,7 @@ namespace WKS4SpreadsheetInternal
 struct Style : public WPSCellFormat
 {
 	//! construtor
-	Style(libwps_tools_win::Font::Type type) : WPSCellFormat(), m_font(), m_fontType(type), m_extra("")
+	explicit Style(libwps_tools_win::Font::Type type) : WPSCellFormat(), m_font(), m_fontType(type), m_extra("")
 	{
 		for (int i = 0; i < 10; i++) m_unknFlags[i] = 0;
 	}
@@ -174,7 +174,7 @@ class Cell : public WPSCell
 {
 public:
 	/// constructor
-	Cell() : m_styleId(-1), m_hAlign(WPSCellFormat::HALIGN_DEFAULT), m_content() { }
+	Cell() : m_styleId(-1), m_hAlign(WPSCellFormat::HALIGN_DEFAULT), m_content(), m_extraTextEntryList() { }
 
 	//! operator<<
 	friend std::ostream &operator<<(std::ostream &o, Cell const &cell);
@@ -199,6 +199,10 @@ public:
 	WPSCellFormat::HorizontalAlignment m_hAlign;
 	//! the content
 	WKSContentListener::CellContent m_content;
+	/** As very long text is splitted in zone 0xf and then in zone 0x36,
+		the list of zone36 text entries...
+	 */
+	std::vector<WPSEntry> m_extraTextEntryList;
 };
 
 //! operator<<
@@ -231,30 +235,13 @@ std::ostream &operator<<(std::ostream &o, Cell const &cell)
 //! the spreadsheet of a WPS4Spreadsheet
 class Spreadsheet
 {
-protected:
-	//! a comparaison structure used to sort cell by rows and and columns
-	struct ComparePosition
-	{
-		//! constructor
-		ComparePosition() {}
-		//! comparaison function
-		bool operator()(Vec2i const &c1, Vec2i const &c2) const
-		{
-			if (c1[1]!=c2[1])
-				return c1[1]<c2[1];
-			return c1[0]<c2[0];
-		}
-	};
-
 public:
 	//! the spreadsheet type
 	enum Type { T_Spreadsheet, T_Filter, T_Report };
 
-	typedef std::map<Vec2i, Cell, ComparePosition> PositionToCellMap_t;
-
 	//! a constructor
-	Spreadsheet(Type type=T_Spreadsheet, int id=0) : m_type(type), m_id(id), m_numCols(0), m_numRows(0), m_LBPosition(-1,-1),
-		m_widthCols(), m_heightRows(), m_positionToCellMap(), m_lastCellPos(),
+	Spreadsheet(Type type=T_Spreadsheet, int id=0) : m_type(type), m_id(id), m_numCols(0), m_numRows(0),
+		m_widthCols(), m_rowHeightMap(), m_heightDefault(16), m_positionToCellMap(), m_lastCellPos(),
 		m_rowPageBreaksList() {}
 	//! return a cell corresponding to a spreadsheet, create one if needed
 	Cell &getCell(Vec2i const &pos)
@@ -283,16 +270,74 @@ public:
 		m_widthCols[size_t(col)] = w;
 		if (col >= m_numCols) m_numCols=col+1;
 	}
-	//! set the rows size
-	void setRowHeight(int row, int h=-1)
-	{
-		if (row < 0) return;
-		if (row >= int(m_heightRows.size())) m_heightRows.resize(size_t(row)+1, -1);
-		m_heightRows[size_t(row)] = h;
-		if (row >= m_numRows) m_numRows=row+1;
-	}
 
-	//! convert the m_widthCols, m_heightRows in a vector of of point size
+	//! returns the row size in point
+	float getRowHeight(int row) const
+	{
+		std::map<Vec2i,int>::const_iterator rIt=m_rowHeightMap.lower_bound(Vec2i(-1,row));
+		if (rIt!=m_rowHeightMap.end() && rIt->first[0]<=row && rIt->first[1]>=row)
+			return (float) rIt->second/20.f;
+		return (float) m_heightDefault;
+	}
+	/** returns the height of a row in point and updated repeated row
+
+		\note: you must first call compressRowHeigths
+	 */
+	float getRowHeight(int row, int &numRepeated) const
+	{
+		std::map<Vec2i,int>::const_iterator rIt=m_rowHeightMap.lower_bound(Vec2i(-1,row));
+		if (rIt!=m_rowHeightMap.end() && rIt->first[0]<=row && rIt->first[1]>=row)
+		{
+			numRepeated=rIt->first[1]-row+1;
+			return (float) rIt->second/20.f;
+		}
+		numRepeated=10000;
+		return (float) m_heightDefault;
+	}
+	//! set the rows size
+	void setRowHeight(int row, int h)
+	{
+		if (h>=0)
+			m_rowHeightMap[Vec2i(row,row)]=h;
+	}
+	//! try to compress the list of row height
+	void compressRowHeights()
+	{
+		std::map<Vec2i,int> oldMap=m_rowHeightMap;
+		m_rowHeightMap.clear();
+		std::map<Vec2i,int>::const_iterator rIt=oldMap.begin();
+		int actHeight=-1;
+		Vec2i actPos(0,-1);
+		int const defHeight=m_heightDefault*20; // conversion from point to TWIP
+		while (rIt!=oldMap.end())
+		{
+			// first check for not filled row
+			if (rIt->first[0]!=actPos[1]+1)
+			{
+				if (actHeight==defHeight)
+					actPos[1]=rIt->first[0]-1;
+				else
+				{
+					if (actPos[1]>=actPos[0])
+						m_rowHeightMap[actPos]=actHeight;
+					actHeight=defHeight;
+					actPos=Vec2i(actPos[1]+1, rIt->first[0]-1);
+				}
+			}
+			if (rIt->second!=actHeight)
+			{
+				if (actPos[1]>=actPos[0])
+					m_rowHeightMap[actPos]=actHeight;
+				actPos[0]=rIt->first[0];
+				actHeight=rIt->second;
+			}
+			actPos[1]=rIt->first[1];
+			++rIt;
+		}
+		if (actPos[1]>=actPos[0])
+			m_rowHeightMap[actPos]=actHeight;
+	}
+	//! convert the m_widthCols in a vector of of point size
 	static std::vector<float> convertInPoint(std::vector<int> const &list,
 	                                         float defSize)
 	{
@@ -306,18 +351,6 @@ public:
 		}
 		return res;
 	}
-	/** compute the last Right Bottom cell position */
-	void computeRightBottomPosition()
-	{
-		int maxX = -1, maxY = -1;
-		for (PositionToCellMap_t::const_iterator it=m_positionToCellMap.begin(); it!=m_positionToCellMap.end(); ++it)
-		{
-			Vec2i const &p = it->second.position();
-			if (p[0] > maxX) maxX = p[0];
-			if (p[1] > maxY) maxY = p[1];
-		}
-		m_LBPosition=Vec2i(maxX, maxY);
-	}
 	//! returns true if the spreedsheet is empty
 	bool empty() const
 	{
@@ -331,15 +364,15 @@ public:
 	int m_numCols;
 	/** the number of rows */
 	int m_numRows;
-	/** the final Right Bottom position, computed by updateState */
-	Vec2i m_LBPosition;
 
 	/** the column size in TWIP (?) */
 	std::vector<int> m_widthCols;
-	/** the row size in TWIP (?) */
-	std::vector<int> m_heightRows;
+	/** the map Vec2i(min row, max row) to size in TWIP (?) */
+	std::map<Vec2i,int> m_rowHeightMap;
+	/** the default row size in point */
+	int m_heightDefault;
 	/** a map cell to not empty cells */
-	PositionToCellMap_t m_positionToCellMap;
+	std::map<Vec2i, Cell> m_positionToCellMap;
 	/** the last cell position */
 	Vec2i m_lastCellPos;
 	/** the list of row page break */
@@ -512,7 +545,6 @@ bool WKS4Spreadsheet::readSheetSize()
 	if (nRow==-1 && nCol==0) return true;
 	if (nRow < 0 || nCol <= 0) return false;
 
-	m_state->getActualSheet().setRowHeight(nRow-1);
 	m_state->getActualSheet().setColumnWidth(nCol-1);
 	return true;
 
@@ -556,7 +588,7 @@ bool WKS4Spreadsheet::readColumnSize()
 			f << "#col[inc],";
 		}
 		// checkme: unit in character(?) -> TWIP
-		m_state->getActualSheet().setColumnWidth(col, width*105);
+		m_state->getActualSheet().setColumnWidth(col, width*160);
 	}
 	ascii().addPos(pos);
 	ascii().addNote(f.str().c_str());
@@ -667,8 +699,8 @@ bool WKS4Spreadsheet::readMsWorksRowSize()
 	int row = libwps::read16(m_input);
 	int width = libwps::readU16(m_input); // unit? 1point ~ 105
 
-	bool ok = row >= 0 && row < m_state->getActualSheet().m_numRows+10;
-	if (ok)m_state->getActualSheet().setRowHeight(row, width & 0x7FFF);
+	bool ok = row >= 0;
+	if (ok) m_state->getActualSheet().setRowHeight(row, width & 0x7FFF);
 	f << "Entries(Row2):Row" << row << ":";
 	if (!ok) f << "###";
 	if (width & 0x8000)
@@ -764,8 +796,10 @@ bool WKS4Spreadsheet::readMsWorksStyle()
 		case 4: // left rellenar
 			style.setHAlignement(WPSCell::HALIGN_LEFT);
 			break;
+		case 5: // center arround cell
+			f << ",center[between cell]";
+		// fail through expected
 		case 2:
-		case 5: // center in selection
 			style.setHAlignement(WPSCell::HALIGN_CENTER);
 			break;
 		case 3:
@@ -1325,7 +1359,7 @@ bool WKS4Spreadsheet::readCell()
 
 	long pos = m_input->tell();
 	long type = libwps::read16(m_input);
-	if ((type != 0x545b) && (type < 0xc || type > 0x10))
+	if ((type != 0x545b) && (type!=0x36) && (type < 0xc || type > 0x10))
 	{
 		WPS_DEBUG_MSG(("WKS4Spreadsheet::readCell: not a cell property\n"));
 		return false;
@@ -1418,7 +1452,14 @@ bool WKS4Spreadsheet::readCell()
 		break;
 	}
 	case 15:
+	case 0x36: // continue text zone
 	{
+		if (type==0x36 && cell.m_content.m_contentType!=WKSContentListener::CellContent::C_TEXT)
+		{
+			WPS_DEBUG_MSG(("WKS4Spreadsheet::readCell: oops, find zone 0x36 but not zone 15\n"));
+			f << "###";
+			break;
+		}
 		cell.m_content.m_contentType=WKSContentListener::CellContent::C_TEXT;
 		long begText=m_input->tell(), endText=begText+dataSz;
 		std::string s("");
@@ -1446,8 +1487,18 @@ bool WKS4Spreadsheet::readCell()
 				--begText;
 			++begText;
 		}
-		cell.m_content.m_textEntry.setBegin(begText);
-		cell.m_content.m_textEntry.setEnd(endText);
+		if (type==15)
+		{
+			cell.m_content.m_textEntry.setBegin(begText);
+			cell.m_content.m_textEntry.setEnd(endText);
+		}
+		else
+		{
+			WPSEntry newEntry;
+			newEntry.setBegin(begText);
+			newEntry.setEnd(endText);
+			cell.m_extraTextEntryList.push_back(newEntry);
+		}
 		break;
 	}
 	case 16:
@@ -2270,30 +2321,37 @@ void WKS4Spreadsheet::sendSpreadsheet(int sId)
 		}
 		sheet.reset(new WKS4SpreadsheetInternal::Spreadsheet);
 	}
-	sheet->computeRightBottomPosition();
 
-	m_listener->openSheet(sheet->convertInPoint(sheet->m_widthCols,76), librevenge::RVNG_POINT,
-	                      m_state->getSheetName(sId));
-	std::vector<float> rowHeight = sheet->convertInPoint(sheet->m_heightRows,16);
-
-	typedef std::map<int, const WKS4SpreadsheetInternal::Cell *> SparseRow_t;
-	typedef std::map<int, SparseRow_t> SparseTable_t;
-	SparseTable_t table;
-	for (WKS4SpreadsheetInternal::Spreadsheet::PositionToCellMap_t::const_iterator it = sheet->m_positionToCellMap.begin(); it != sheet->m_positionToCellMap.end(); ++it)
-		table[it->first[1]][it->first[0]] = &it->second;
-
-	SparseTable_t::const_iterator rIt = table.begin();
-	for (int r=0; r<=sheet->m_LBPosition[1] && rIt != table.end(); ++r)
+	m_listener->openSheet(sheet->convertInPoint(sheet->m_widthCols,72), librevenge::RVNG_POINT,
+	                      std::vector<int>(), m_state->getSheetName(sId));
+	sheet->compressRowHeights();
+	std::map<Vec2i, WKS4SpreadsheetInternal::Cell>::const_iterator it = sheet->m_positionToCellMap.begin();
+	int prevRow = -1;
+	while (it!=sheet->m_positionToCellMap.end())
 	{
-		m_listener->openSheetRow(r < int(rowHeight.size()) ? rowHeight[size_t(r)] : 16, librevenge::RVNG_POINT);
-		if (r == rIt->first) // a row with at least one non-empty cell
+		int row=it->first[1];
+		WKS4SpreadsheetInternal::Cell const &cell=(it++)->second;
+		if (row>prevRow+1)
 		{
-			for (SparseRow_t::const_iterator cIt = rIt->second.begin(); cIt != rIt->second.end(); ++cIt)
-				sendCellContent(*cIt->second);
-			++rIt;
+			while (row > prevRow+1)
+			{
+				if (prevRow != -1) m_listener->closeSheetRow();
+				int numRepeat;
+				float h=sheet->getRowHeight(prevRow+1, numRepeat);
+				if (row<prevRow+1+numRepeat)
+					numRepeat=row-1-prevRow;
+				m_listener->openSheetRow(h, librevenge::RVNG_POINT, false, numRepeat);
+				prevRow+=numRepeat;
+			}
 		}
-		m_listener->closeSheetRow();
+		if (row!=prevRow)
+		{
+			if (prevRow != -1) m_listener->closeSheetRow();
+			m_listener->openSheetRow(sheet->getRowHeight(++prevRow), librevenge::RVNG_POINT);
+		}
+		sendCellContent(cell);
 	}
+	if (prevRow!=-1) m_listener->closeSheetRow();
 	m_listener->closeSheet();
 }
 
@@ -2313,7 +2371,6 @@ void WKS4Spreadsheet::sendCellContent(WKS4SpreadsheetInternal::Cell const &cell)
 	if (version()<=2 && cell.m_hAlign!=WPSCellFormat::HALIGN_DEFAULT)
 		cellStyle.setHAlignement(cell.m_hAlign);
 
-	librevenge::RVNGPropertyList propList;
 	libwps_tools_win::Font::Type fontType = cellStyle.m_fontType;
 	m_listener->setFont(cellStyle.m_font);
 
@@ -2331,21 +2388,26 @@ void WKS4Spreadsheet::sendCellContent(WKS4SpreadsheetInternal::Cell const &cell)
 		for (size_t c=0; c < text.length(); ++c)
 		{
 			if (!hasLICS)
-				WPSListener::appendUnicode
+				libwps::appendUnicode
 				((uint32_t)libwps_tools_win::Font::unicode((unsigned char)text[c],fontType), finalString);
 			else
-				WPSListener::appendUnicode
+				libwps::appendUnicode
 				((uint32_t)libwps_tools_win::Font::LICSunicode((unsigned char)text[c],fontType), finalString);
 		}
 		text=finalString.cstr();
 	}
-	m_listener->openSheetCell(finalCell, content, propList);
+	m_listener->openSheetCell(finalCell, content);
 
-	if (cell.m_content.m_textEntry.valid())
+	size_t numTextEntry=cell.m_content.m_textEntry.valid() ? 1 : 0;
+	if (!cell.m_extraTextEntryList.empty())
+		numTextEntry=1+cell.m_extraTextEntryList.size();
+	for (size_t t=0; t<numTextEntry; ++t)
 	{
-		m_input->seek(cell.m_content.m_textEntry.begin(), librevenge::RVNG_SEEK_SET);
+		WPSEntry entry=t==0 ? cell.m_content.m_textEntry : cell.m_extraTextEntryList[t-1];
+		if (!entry.valid()) continue;
+		m_input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
 		bool prevEOL=false;
-		while (!m_input->isEnd() && m_input->tell()<cell.m_content.m_textEntry.end())
+		while (!m_input->isEnd() && m_input->tell()<entry.end())
 		{
 			unsigned char c=(unsigned char) libwps::readU8(m_input);
 			if (c==0xd)
